@@ -1,7 +1,8 @@
 #!/bin/sh
-srcversion='$MirOS: src/bin/mksh/Build.sh,v 1.484.2.7 2011/12/11 18:18:18 tg Exp $'
+srcversion='$MirOS: src/bin/mksh/Build.sh,v 1.549 2012/04/14 16:07:43 tg Exp $'
 #-
-# Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
+#		2011, 2012
 #	Thorsten Glaser <tg@mirbsd.org>
 #
 # Provided that these terms and disclaimer and all copyright notices
@@ -27,6 +28,17 @@ srcversion='$MirOS: src/bin/mksh/Build.sh,v 1.484.2.7 2011/12/11 18:18:18 tg Exp
 LC_ALL=C
 export LC_ALL
 
+if test -n "${ZSH_VERSION+x}" && (emulate sh) >/dev/null 2>&1; then
+	emulate sh
+	NULLCMD=:
+fi
+
+if test -d /usr/xpg4/bin/. >/dev/null 2>&1; then
+	# Solaris: some of the tools have weird behaviour, use portable ones
+	PATH=/usr/xpg4/bin:$PATH
+	export PATH
+fi
+
 v() {
 	$e "$*"
 	eval "$@"
@@ -53,17 +65,6 @@ rmf() {
 	done
 }
 
-if test -d /usr/xpg4/bin/. >/dev/null 2>&1; then
-	# Solaris: some of the tools have weird behaviour, use portable ones
-	PATH=/usr/xpg4/bin:$PATH
-	export PATH
-fi
-
-if test -n "${ZSH_VERSION+x}" && (emulate sh) >/dev/null 2>&1; then
-	emulate sh
-	NULLCMD=:
-fi
-
 allu=QWERTYUIOPASDFGHJKLZXCVBNM
 alll=qwertyuiopasdfghjklzxcvbnm
 alln=0123456789
@@ -147,14 +148,14 @@ ac_testinit() {
 }
 
 # pipe .c | ac_test[n] [!] label [!] checkif[!]0 [setlabelifcheckis[!]0] useroutput
-ac_testn() {
+ac_testnnd() {
 	if test x"$1" = x"!"; then
 		fr=1
 		shift
 	else
 		fr=0
 	fi
-	ac_testinit "$@" || return
+	ac_testinit "$@" || return 1
 	cat >conftest.c
 	vv ']' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN conftest.c $LIBS $ccpr"
 	test $tcfn = no && test -f a.out && tcfn=a.out
@@ -173,6 +174,10 @@ ac_testn() {
 		test $ct = sunpro && vscan='-e ignored -e turned.off'
 	fi
 	test -n "$vscan" && grep $vscan vv.out >/dev/null 2>&1 && fv=$fr
+	return 0
+}
+ac_testn() {
+	ac_testnnd "$@" || return
 	rmf conftest.c conftest.o ${tcfn}* vv.out
 	ac_testdone
 }
@@ -258,6 +263,7 @@ ac_header() {
 	fi
 	hf=$1; shift
 	hv=`echo "$hf" | tr -d '\012\015' | tr -c $alll$allu$alln $alls`
+	echo "/* NeXTstep bug workaround */" >x
 	for i
 	do
 		echo "#include <$i>" >>x
@@ -270,6 +276,12 @@ ac_header() {
 }
 
 addsrcs() {
+	addsrcs_s=0
+	if test x"$1" = x"-s"; then
+		# optstatic
+		addsrcs_s=1
+		shift
+	fi
 	if test x"$1" = x"!"; then
 		fr=0
 		shift
@@ -277,6 +289,13 @@ addsrcs() {
 		fr=1
 	fi
 	eval i=\$$1
+	if test $addsrcs_s = 1; then
+		if test -f "$2" || test -f "$srcdir/$2"; then
+			# always add $2, since it exists
+			fr=1
+			i=1
+		fi
+	fi
 	test $fr = "$i" && case " $SRCS " in
 	*\ $2\ *)	;;
 	*)		SRCS="$SRCS $2" ;;
@@ -288,12 +307,13 @@ if test -d mksh || test -d mksh.exe; then
 	echo "$me: Error: ./mksh is a directory!" >&2
 	exit 1
 fi
-rmf a.exe* a.out* conftest.c *core lft mksh* no *.bc *.ll *.o \
+rmf a.exe* a.out* conftest.c *core core.* lft mksh* no *.bc *.ll *.o \
     Rebuild.sh signames.inc test.sh x vv.out
 
-curdir=`pwd` srcdir=`dirname "$0"` check_categories=
-test -n "$srcdir" || srcdir=.
+curdir=`pwd` srcdir=`dirname "$0" 2>/dev/null` check_categories=
+test -n "$srcdir" || srcdir=. # in case dirname does not exist
 dstversion=`sed -n '/define MKSH_VERSION/s/^.*"\(.*\)".*$/\1/p' $srcdir/sh.h`
+add_cppflags -DMKSH_BUILDSH
 
 e=echo
 r=0
@@ -321,10 +341,6 @@ do
 	:-c)
 		last=c
 		;;
-	:-combine)
-		cm=combine
-		echo "$me: Warning: '$i' is deprecated, use '-c combine' instead!" >&2
-		;;
 	:-g)
 		# checker, debug, valgrind build
 		add_cppflags -DDEBUG
@@ -333,16 +349,6 @@ do
 	:-j)
 		pm=1
 		;;
-	:-llvm)
-		cm=llvm
-		optflags=-std-compile-opts
-		echo "$me: Warning: '$i' is deprecated, use '-c llvm -O' instead!" >&2
-		;;
-	:-llvm=*)
-		cm=llvm
-		optflags=`echo "x$i" | sed 's/^x-llvm=//'`
-		echo "$me: Warning: '$i' is deprecated, use '-c llvm -o $llvm' instead!" >&2
-		;;
 	:-M)
 		cm=makefile
 		;;
@@ -386,6 +392,11 @@ if test x"$srcdir" = x"."; then
 else
 	CPPFLAGS="-I. -I'$srcdir' $CPPFLAGS"
 fi
+test -n "$LDSTATIC" && if test -n "$LDFLAGS"; then
+	LDFLAGS="$LDFLAGS $LDSTATIC"
+else
+	LDFLAGS=$LDSTATIC
+fi
 
 test x"$TARGET_OS" = x"" && TARGET_OS=`uname -s 2>/dev/null || uname`
 if test x"$TARGET_OS" = x""; then
@@ -404,21 +415,86 @@ if test x"$TARGET_OS" = x"Android"; then
 	TARGET_OS=Linux
 fi
 
+# Evil OS
+if test x"$TARGET_OS" = x"Minix"; then
+	echo >&2 "
+WARNING: additional checks before running Build.sh required!
+You can avoid these by calling Build.sh correctly, see below.
+"
+	cat >conftest.c <<'EOF'
+#include <sys/types.h>
+const char *
+#ifdef _NETBSD_SOURCE
+ct="Ninix3"
+#else
+ct="Minix3"
+#endif
+;
+EOF
+	ct=unknown
+	vv ']' "${CC-cc} -E $CFLAGS $CPPFLAGS $NOWARN conftest.c | grep ct= | tr -d \\\\015 >x"
+	sed 's/^/[ /' x
+	eval `cat x`
+	rmf x vv.out
+	case $ct in
+	Minix3|Ninix3)
+		echo >&2 "
+Warning: you set TARGET_OS to $TARGET_OS but that is ambiguous.
+Please set it to either Minix3 or Ninix3, whereas the latter is
+all versions of Minix with even partial NetBSD(R) userland. The
+value determined from your compiler for the current compilation
+(which may be wrong) is: $ct
+"
+		TARGET_OS=$ct
+		;;
+	*)
+		echo >&2 "
+Warning: you set TARGET_OS to $TARGET_OS but that is ambiguous.
+Please set it to either Minix3 or Ninix3, whereas the latter is
+all versions of Minix with even partial NetBSD(R) userland. The
+proper value couldn't be determined, continue at your own risk.
+"
+		;;
+	esac
+fi
+
 # Configuration depending on OS revision, on OSes that need them
 case $TARGET_OS in
-QNX)
+NEXTSTEP)
+	test x"$TARGET_OSREV" = x"" && TARGET_OSREV=`hostinfo 2>&1 | sed -n '/^.*NeXT Mach \([0-9.]*\):.*$/s//\1/p'`
+	;;
+QNX|SCO_SV)
 	test x"$TARGET_OSREV" = x"" && TARGET_OSREV=`uname -r`
 	;;
 esac
 
 # Configuration depending on OS name
 case $TARGET_OS in
+386BSD)
+	# required fixes for 386BSD-0.0new are:
+	# http://groups.google.com/group/comp.unix.bsd/browse_thread/thread/1c6397039f10e76b?hl=en
+	# http://groups.google.com/group/comp.unix.bsd/browse_thread/thread/cb899f7ccb81550b?hl=en
+	# from 386BSD-0.1 on, it works out of the box
+	: ${HAVE_CAN_OTWO=0}
+	add_cppflags -DMKSH_NO_SIGSETJMP
+	add_cppflags -DMKSH_TYPEDEF_SIG_ATOMIC_T=int
+	;;
 AIX)
 	add_cppflags -D_ALL_SOURCE
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 BeOS)
-	oswarn=' and will currently not work'
+	case $KSH_VERSION in
+	*MIRBSD\ KSH*)
+		oswarn="; it has minor issues"
+		;;
+	*)
+		oswarn="; you must recompile mksh with"
+		oswarn="$oswarn${nl}itself in a second stage"
+		;;
+	esac
+	# BeOS has no real tty either
+	add_cppflags -DMKSH_UNEMPLOYED
 	;;
 BSD/OS)
 	: ${HAVE_SETLOCALE_CTYPE=0}
@@ -452,7 +528,7 @@ GNU/kFreeBSD)
 	esac
 	;;
 Haiku)
-	add_cppflags -DMKSH_ASSUME_UTF8
+	add_cppflags -DMKSH_ASSUME_UTF8; HAVE_ISSET_MKSH_ASSUME_UTF8=1
 	;;
 HP-UX)
 	;;
@@ -474,9 +550,12 @@ Linux)
 	add_cppflags -DSETUID_CAN_FAIL_WITH_EAGAIN
 	: ${HAVE_REVOKE=0}
 	;;
+LynxOS)
+	oswarn="; it has minor issues"
+	;;
 MidnightBSD)
 	;;
-Minix)
+Minix3)
 	add_cppflags -DMKSH_UNEMPLOYED
 	add_cppflags -DMKSH_CONSERVATIVE_FDS
 	add_cppflags -DMKSH_NO_LIMITS
@@ -487,6 +566,7 @@ Minix)
 MirBSD)
 	;;
 MSYS_*)
+	add_cppflags -DMKSH_ASSUME_UTF8=0; HAVE_ISSET_MKSH_ASSUME_UTF8=1
 	# almost same as CYGWIN* (from RT|Chatzilla)
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	# broken on this OE (from ir0nh34d)
@@ -494,6 +574,28 @@ MSYS_*)
 	;;
 NetBSD)
 	;;
+NEXTSTEP)
+	add_cppflags -D_NEXT_SOURCE
+	add_cppflags -D_POSIX_SOURCE
+	: ${AWK=gawk} ${CC=cc -posix}
+	add_cppflags -DMKSH_NO_SIGSETJMP
+	# NeXTstep cannot get a controlling tty
+	add_cppflags -DMKSH_UNEMPLOYED
+	case $TARGET_OSREV in
+	4.2*)
+		# OpenStep 4.2 is broken by default
+		oswarn="; it needs libposix.a"
+		;;
+	esac
+	;;
+Ninix3)
+	# similar to Minix3
+	add_cppflags -DMKSH_UNEMPLOYED
+	add_cppflags -DMKSH_CONSERVATIVE_FDS
+	add_cppflags -DMKSH_NO_LIMITS
+	# but no idea what else could be needed
+	oswarn="; it has unknown issues"
+	;;
 OpenBSD)
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
@@ -510,7 +612,7 @@ Plan9)
 	add_cppflags -D_LIMITS_EXTENSION
 	add_cppflags -D_BSD_EXTENSION
 	add_cppflags -D_SUSV2_SOURCE
-	add_cppflags -DMKSH_ASSUME_UTF8
+	add_cppflags -DMKSH_ASSUME_UTF8; HAVE_ISSET_MKSH_ASSUME_UTF8=1
 	oswarn=' and will currently not work'
 	add_cppflags -DMKSH_UNEMPLOYED
 	;;
@@ -528,6 +630,25 @@ QNX)
 	esac
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
+SCO_SV)
+	case $TARGET_OSREV in
+	3.2*)
+		# SCO OpenServer 5
+		add_cppflags -DMKSH_UNEMPLOYED
+		;;
+	5*)
+		# SCO OpenServer 6
+		;;
+	*)
+		oswarn='; this is an unknown version of'
+		oswarn="$oswarn$nl$TARGET_OS ${TARGET_OSREV}, please tell me what to do"
+		;;
+	esac
+	: ${HAVE_SYS_SIGLIST=0} ${HAVE__SYS_SIGLIST=0}
+	;;
+skyos)
+	oswarn="; it has minor issues"
+	;;
 SunOS)
 	add_cppflags -D_BSD_SOURCE
 	add_cppflags -D__EXTENSIONS__
@@ -538,9 +659,13 @@ syllable)
 	;;
 ULTRIX)
 	: ${CC=cc -YPOSIX}
-	add_cppflags -Dssize_t=int
+	add_cppflags -DMKSH_TYPEDEF_SSIZE_T=int
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
+UnixWare|UNIX_SV)
+	# SCO UnixWare
+	: ${HAVE_SYS_SIGLIST=0} ${HAVE__SYS_SIGLIST=0}
+	;;
 UWIN*)
 	ccpc='-Yc,'
 	ccpl='-Yl,'
@@ -551,12 +676,13 @@ UWIN*)
 	;;
 *)
 	oswarn='; it may or may not work'
+	test x"$TARGET_OSREV" = x"" && TARGET_OSREV=`uname -r`
 	;;
 esac
 
 : ${HAVE_MKNOD=0}
 
-: ${CC=cc} ${NROFF=nroff}
+: ${AWK=awk} ${CC=cc} ${NROFF=nroff}
 test 0 = $r && echo | $NROFF -v 2>&1 | grep GNU >/dev/null 2>&1 && \
     NROFF="$NROFF -c"
 
@@ -579,6 +705,10 @@ OSF1)
 	vv '|' "uname -a >&2"
 	vv '|' "/usr/sbin/sizer -v >&2"
 	;;
+SCO_SV|UnixWare|UNIX_SV)
+	vv '|' "uname -a >&2"
+	vv '|' "uname -X >&2"
+	;;
 *)
 	vv '|' "uname -a >&2"
 	;;
@@ -589,7 +719,7 @@ operating system '$TARGET_OS'$oswarn. If you can provide
 a shell account to the developer, this may improve; please
 drop us a success or failure notice or even send in diffs.
 "
-$e "$bi$me: Building the MirBSD Korn Shell$ao $ui$dstversion$ao"
+$e "$bi$me: Building the MirBSD Korn Shell$ao $ui$dstversion$ao on $TARGET_OS ${TARGET_OSREV}..."
 
 #
 # Begin of mirtoconf checks
@@ -607,62 +737,66 @@ $e $bi$me: Scanning for functions... please ignore any errors.$ao
 CPP="$CC -E"
 $e ... which compiler seems to be used
 cat >conftest.c <<'EOF'
+const char *
 #if defined(__ICC) || defined(__INTEL_COMPILER)
-ct=icc
+ct="icc"
 #elif defined(__xlC__) || defined(__IBMC__)
-ct=xlc
+ct="xlc"
 #elif defined(__SUNPRO_C)
-ct=sunpro
+ct="sunpro"
 #elif defined(__ACK__)
-ct=ack
+ct="ack"
 #elif defined(__BORLANDC__)
-ct=bcc
+ct="bcc"
 #elif defined(__WATCOMC__)
-ct=watcom
+ct="watcom"
 #elif defined(__MWERKS__)
-ct=metrowerks
+ct="metrowerks"
 #elif defined(__HP_cc)
-ct=hpcc
+ct="hpcc"
 #elif defined(__DECC) || (defined(__osf__) && !defined(__GNUC__))
-ct=dec
+ct="dec"
 #elif defined(__PGI)
-ct=pgi
+ct="pgi"
 #elif defined(__DMC__)
-ct=dmc
+ct="dmc"
 #elif defined(_MSC_VER)
-ct=msc
+ct="msc"
 #elif defined(__ADSPBLACKFIN__) || defined(__ADSPTS__) || defined(__ADSP21000__)
-ct=adsp
+ct="adsp"
 #elif defined(__IAR_SYSTEMS_ICC__)
-ct=iar
+ct="iar"
 #elif defined(SDCC)
-ct=sdcc
+ct="sdcc"
 #elif defined(__PCC__)
-ct=pcc
+ct="pcc"
 #elif defined(__TenDRA__)
-ct=tendra
+ct="tendra"
 #elif defined(__TINYC__)
-ct=tcc
+ct="tcc"
 #elif defined(__llvm__) && defined(__clang__)
-ct=clang
+ct="clang"
 #elif defined(__NWCC__)
-ct=nwcc
+ct="nwcc"
 #elif defined(__GNUC__)
-ct=gcc
+ct="gcc"
 #elif defined(_COMPILER_VERSION)
-ct=mipspro
+ct="mipspro"
 #elif defined(__sgi)
-ct=mipspro
+ct="mipspro"
 #elif defined(__hpux) || defined(__hpua)
-ct=hpcc
+ct="hpcc"
 #elif defined(__ultrix)
-ct=ucode
+ct="ucode"
+#elif defined(__USLC__)
+ct="uslc"
 #else
-ct=unknown
+ct="unknown"
 #endif
+;
 EOF
-ct=unknown
-vv ']' "$CPP $CFLAGS $CPPFLAGS $NOWARN conftest.c | grep ct= | tr -d \\\\015 >x"
+ct=untested
+vv ']' "$CPP $CFLAGS $CPPFLAGS $NOWARN conftest.c | sed -n '/^ct *= */s//ct=/p' | tr -d \\\\015 >x"
 sed 's/^/[ /' x
 eval `cat x`
 rmf x vv.out
@@ -686,8 +820,9 @@ bcc)
 clang)
 	# does not work with current "ccc" compiler driver
 	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN $LIBS -version"
-	# this works, for now
+	# one of these two works, for now
 	vv '|' "${CLANG-clang} -version"
+	vv '|' "${CLANG-clang} --version"
 	# ensure compiler and linker are in sync unless overridden
 	case $CCC_CC:$CCC_LD in
 	:*)	;;
@@ -780,6 +915,16 @@ ucode)
 	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN $LIBS -V"
 	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -Wl,-V conftest.c $LIBS"
 	;;
+uslc)
+	case $TARGET_OS:$TARGET_OSREV in
+	SCO_SV:3.2*)
+		# SCO OpenServer 5
+		CFLAGS="$CFLAGS -g"
+		: ${HAVE_CAN_OTWO=0} ${HAVE_CAN_OPTIMISE=0}
+		;;
+	esac
+	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -V conftest.c $LIBS"
+	;;
 watcom)
 	echo >&2 'Warning: Watcom C Compiler detected. This compiler has not yet
     been tested for compatibility with mksh. Continue at your
@@ -791,7 +936,11 @@ xlc)
 	vv '|' "ld -V"
 	;;
 *)
+	test x"$ct" = x"untested" && $e "!!! detecting preprocessor failed"
 	ct=unknown
+	vv "$CC --version"
+	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -v conftest.c $LIBS"
+	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -V conftest.c $LIBS"
 	;;
 esac
 case $cm in
@@ -838,6 +987,7 @@ if ac_ifcpp 'ifdef __TINYC__' couldbe_tcc '!' compiler_known 0 \
     'if this could be tcc'; then
 	ct=tcc
 	CPP='cpp -D__TINYC__'
+	HAVE_COMPILER_KNOWN=1
 fi
 
 if test $ct = sunpro; then
@@ -918,7 +1068,6 @@ if test $ct = gcc; then
 	ac_flags 0 wnooverflow -Wno-overflow
 	ac_flags 1 fnostrictaliasing -fno-strict-aliasing
 	ac_flags 1 fstackprotectorall -fstack-protector-all
-	ac_flags 1 fwrapv -fwrapv
 	test $cm = dragonegg && case " $CC $CFLAGS $LDFLAGS " in
 	*\ -fplugin=*dragonegg*) ;;
 	*) ac_flags 1 fplugin_dragonegg -fplugin=dragonegg ;;
@@ -968,13 +1117,12 @@ elif test $ct = icc; then
 elif test $ct = sunpro; then
 	phase=u
 	ac_flags 1 v -v
-	ac_flags 1 xc99 -xc99 'for support of ISO C99'
 	ac_flags 1 ipo -xipo 'for cross-module optimisation'
 	phase=x
 elif test $ct = hpcc; then
 	phase=u
-	ac_flags 1 agcc -Agcc 'for support of GCC extensions'
-	ac_flags 1 ac99 -AC99 'for support of ISO C99'
+	# probably not needed
+	#ac_flags 1 agcc -Agcc 'for support of GCC extensions'
 	phase=x
 elif test $ct = dec; then
 	ac_flags 0 verb -verbose
@@ -985,7 +1133,6 @@ elif test $ct = dmc; then
 elif test $ct = bcc; then
 	ac_flags 1 strpool "${ccpc}-d" 'if string pooling can be enabled'
 elif test $ct = mipspro; then
-	ac_flags 1 xc99 -c99 'for support of ISO C99'
 	ac_flags 1 fullwarn -fullwarn 'for remark output support'
 elif test $ct = msc; then
 	ac_flags 1 strpool "${ccpc}/GF" 'if string pooling can be enabled'
@@ -997,8 +1144,6 @@ elif test $ct = msc; then
 	ac_flags 1 wall "${ccpc}/Wall" 'to enable all warnings'
 	ac_flags 1 wp64 "${ccpc}/Wp64" 'to enable 64-bit warnings'
 elif test $ct = xlc; then
-	ac_flags 1 x99 -qlanglvl=extc99
-	test 1 = $HAVE_CAN_X99 || ac_flags 1 c99 -qlanglvl=stdc99
 	ac_flags 1 rodata "-qro -qroconst -qroptr"
 	ac_flags 1 rtcheck -qcheck=all
 	ac_flags 1 rtchkc -qextchk
@@ -1009,19 +1154,17 @@ elif test $ct = tendra; then
 	test 1 = $HAVE_CAN_YSYSTEM && CPPFLAGS="-Ysystem $CPPFLAGS"
 	ac_flags 1 extansi -Xa
 elif test $ct = tcc; then
-	ac_flags 1 boundschk -b
+	: #broken# ac_flags 1 boundschk -b
 elif test $ct = clang; then
 	i=1
 elif test $ct = nwcc; then
 	i=1
-	#broken# ac_flags 1 ssp -stackprotect
+	: #broken# ac_flags 1 ssp -stackprotect
 fi
 # flags common to a subset of compilers (run with -Werror on gcc)
 if test 1 = $i; then
-	ac_flags 1 stdg99 -std=gnu99 'for support of ISO C99 + GCC extensions'
-	test 1 = $HAVE_CAN_STDG99 || \
-	    ac_flags 1 stdc99 -std=c99 'for support of ISO C99'
 	ac_flags 1 wall -Wall
+	ac_flags 1 fwrapv -fwrapv
 fi
 
 phase=x
@@ -1116,24 +1259,6 @@ phase=x
 #
 if ac_ifcpp 'ifdef MKSH_SMALL' isset_MKSH_SMALL '' \
     "if a reduced-feature mksh is requested"; then
-	#XXX this sucks; fix it for *all* compilers
-	case $ct in
-	clang|icc|nwcc)
-		ac_flags 1 fnoinline -fno-inline
-		;;
-	gcc)
-		NOWARN=$DOWARN; phase=u
-		ac_flags 1 fnoinline -fno-inline
-		NOWARN=$save_NOWARN; phase=x
-		;;
-	sunpro)
-		ac_flags 1 fnoinline -xinline=
-		;;
-	xlc)
-		ac_flags 1 fnoinline -qnoinline
-		;;
-	esac
-
 	: ${HAVE_NICE=0}
 	: ${HAVE_PERSISTENT_HISTORY=0}
 	check_categories="$check_categories smksh"
@@ -1153,9 +1278,9 @@ ac_ifcpp 'ifdef MKSH_ASSUME_UTF8' isset_MKSH_ASSUME_UTF8 '' \
 ac_ifcpp 'ifdef MKSH_CONSERVATIVE_FDS' isset_MKSH_CONSERVATIVE_FDS '' \
     'if traditional/conservative fd use is requested' && \
     check_categories="$check_categories convfds"
-ac_ifcpp 'ifdef MKSH_DISABLE_DEPRECATED' isset_MKSH_DISABLE_DEPRECATED '' \
-    "if deprecated features are to be omitted" && \
-    check_categories="$check_categories nodeprecated"
+#ac_ifcpp 'ifdef MKSH_DISABLE_DEPRECATED' isset_MKSH_DISABLE_DEPRECATED '' \
+#    "if deprecated features are to be omitted" && \
+#    check_categories="$check_categories nodeprecated"
 
 #
 # Environment: headers
@@ -1178,51 +1303,6 @@ ac_header strings.h sys/types.h string.h
 ac_header ulimit.h sys/types.h
 ac_header values.h
 
-#
-# check whether whatever we use for the final link will succeed
-#
-if test $cm = makefile; then
-	: nothing to check
-else
-	HAVE_LINK_WORKS=x
-	ac_testinit link_works '' 'checking if the final link command may succeed'
-	fv=1
-	cat >conftest.c <<-'EOF'
-		#define EXTERN
-		#define MKSH_INCLUDES_ONLY
-		#include "sh.h"
-		__RCSID("$MirOS: src/bin/mksh/Build.sh,v 1.484.2.7 2011/12/11 18:18:18 tg Exp $");
-		int main(void) { printf("Hello, World!\n"); return (0); }
-EOF
-	case $cm in
-	llvm)
-		v "$CC $CFLAGS $CPPFLAGS $NOWARN -emit-llvm -c conftest.c" || fv=0
-		rmf mksh.s
-		test $fv = 0 || v "llvm-link -o - conftest.o | opt $optflags | llc -o mksh.s" || fv=0
-		test $fv = 0 || v "$CC $CFLAGS $LDFLAGS -o $tcfn mksh.s $LIBS $ccpr"
-		;;
-	dragonegg)
-		v "$CC $CFLAGS $CPPFLAGS $NOWARN -S -flto conftest.c" || fv=0
-		test $fv = 0 || v "mv conftest.s conftest.ll"
-		test $fv = 0 || v "llvm-as conftest.ll" || fv=0
-		rmf mksh.s
-		test $fv = 0 || v "llvm-link -o - conftest.bc | opt $optflags | llc -o mksh.s" || fv=0
-		test $fv = 0 || v "$CC $CFLAGS $LDFLAGS -o $tcfn mksh.s $LIBS $ccpr"
-		;;
-	combine)
-		v "$CC $CFLAGS $CPPFLAGS $LDFLAGS -fwhole-program --combine $NOWARN -o $tcfn conftest.c $LIBS $ccpr"
-		;;
-	lto|normal)
-		cm=normal
-		v "$CC $CFLAGS $CPPFLAGS $NOWARN -c conftest.c" || fv=0
-		test $fv = 0 || v "$CC $CFLAGS $LDFLAGS -o $tcfn conftest.o $LIBS $ccpr"
-		;;
-	esac
-	test -f $tcfn || fv=0
-	ac_testdone
-	test $fv = 1 || exit 1
-fi
-
 #
 # Environment: definitions
 #
@@ -1309,6 +1389,51 @@ fi
 test 1 = $HAVE_SIG_T || add_cppflags -Dsig_t=nosig_t
 ac_cppflags SIG_T
 
+#
+# check whether whatever we use for the final link will succeed
+#
+if test $cm = makefile; then
+	: nothing to check
+else
+	HAVE_LINK_WORKS=x
+	ac_testinit link_works '' 'checking if the final link command may succeed'
+	fv=1
+	cat >conftest.c <<-'EOF'
+		#define EXTERN
+		#define MKSH_INCLUDES_ONLY
+		#include "sh.h"
+		__RCSID("$MirOS: src/bin/mksh/Build.sh,v 1.549 2012/04/14 16:07:43 tg Exp $");
+		int main(void) { printf("Hello, World!\n"); return (0); }
+EOF
+	case $cm in
+	llvm)
+		v "$CC $CFLAGS $CPPFLAGS $NOWARN -emit-llvm -c conftest.c" || fv=0
+		rmf mksh.s
+		test $fv = 0 || v "llvm-link -o - conftest.o | opt $optflags | llc -o mksh.s" || fv=0
+		test $fv = 0 || v "$CC $CFLAGS $LDFLAGS -o $tcfn mksh.s $LIBS $ccpr"
+		;;
+	dragonegg)
+		v "$CC $CFLAGS $CPPFLAGS $NOWARN -S -flto conftest.c" || fv=0
+		test $fv = 0 || v "mv conftest.s conftest.ll"
+		test $fv = 0 || v "llvm-as conftest.ll" || fv=0
+		rmf mksh.s
+		test $fv = 0 || v "llvm-link -o - conftest.bc | opt $optflags | llc -o mksh.s" || fv=0
+		test $fv = 0 || v "$CC $CFLAGS $LDFLAGS -o $tcfn mksh.s $LIBS $ccpr"
+		;;
+	combine)
+		v "$CC $CFLAGS $CPPFLAGS $LDFLAGS -fwhole-program --combine $NOWARN -o $tcfn conftest.c $LIBS $ccpr"
+		;;
+	lto|normal)
+		cm=normal
+		v "$CC $CFLAGS $CPPFLAGS $NOWARN -c conftest.c" || fv=0
+		test $fv = 0 || v "$CC $CFLAGS $LDFLAGS -o $tcfn conftest.o $LIBS $ccpr"
+		;;
+	esac
+	test -f $tcfn || fv=0
+	ac_testdone
+	test $fv = 1 || exit 1
+fi
+
 #
 # Environment: signals
 #
@@ -1341,19 +1466,20 @@ EOF
 #
 # Environment: library functions
 #
-ac_testn flock_ex '' 'flock and mmap' <<-'EOF'
-	#include <sys/types.h>
-	#if HAVE_SYS_FILE_H
-	#include <sys/file.h>
-	#endif
-	#if HAVE_SYS_MMAN_H
-	#include <sys/mman.h>
-	#endif
+ac_test flock <<-'EOF'
 	#include <fcntl.h>
-	#include <stdlib.h>
-	int main(void) { return ((void *)mmap(NULL, (size_t)flock(0, LOCK_EX),
-	    PROT_READ, MAP_PRIVATE, 0, (off_t)0) == (void *)NULL ? 1 :
-	    munmap(NULL, 0)); }
+	#undef flock
+	int main(void) { return (flock(0, LOCK_EX | LOCK_UN)); }
+EOF
+
+ac_test lock_fcntl '!' flock 1 'whether we can lock files with fcntl' <<-'EOF'
+	#include <fcntl.h>
+	#undef flock
+	int main(void) {
+		struct flock lks;
+		lks.l_type = F_WRLCK | F_UNLCK;
+		return (fcntl(0, F_SETLKW, &lks));
+	}
 EOF
 
 ac_test getrusage <<-'EOF'
@@ -1382,10 +1508,19 @@ ac_test mknod '' 'if to use mknod(), makedev() and friends' <<-'EOF'
 	}
 EOF
 
-ac_test mkstemp <<-'EOF'
+ac_test mmap lock_fcntl 0 'for mmap and munmap' <<-'EOF'
+	#include <sys/types.h>
+	#if HAVE_SYS_FILE_H
+	#include <sys/file.h>
+	#endif
+	#if HAVE_SYS_MMAN_H
+	#include <sys/mman.h>
+	#endif
+	#include <stddef.h>
 	#include <stdlib.h>
-	#include <unistd.h>
-	int main(void) { char tmpl[] = "X"; return (mkstemp(tmpl)); }
+	int main(void) { return ((void *)mmap(NULL, (size_t)0,
+	    PROT_READ, MAP_PRIVATE, 0, (off_t)0) == (void *)NULL ? 1 :
+	    munmap(NULL, 0)); }
 EOF
 
 ac_test nice <<-'EOF'
@@ -1455,18 +1590,6 @@ ac_test setgroups setresugid 0 <<-'EOF'
 	int main(void) { gid_t gid = 0; return (setgroups(0, &gid)); }
 EOF
 
-ac_test strcasestr <<-'EOF'
-	#include <sys/types.h>
-	#include <stddef.h>
-	#include <string.h>
-	#if HAVE_STRINGS_H
-	#include <strings.h>
-	#endif
-	int main(int ac, char *av[]) {
-		return ((int)(ptrdiff_t)(void *)strcasestr(*av, av[ac]));
-	}
-EOF
-
 ac_test strlcpy <<-'EOF'
 	#include <string.h>
 	int main(int ac, char *av[]) { return (strlcpy(*av, av[1],
@@ -1478,7 +1601,7 @@ EOF
 #
 save_CC=$CC; save_LDFLAGS=$LDFLAGS; save_LIBS=$LIBS
 CC="$CC -c -o $tcfn"; LDFLAGS=; LIBS=
-ac_test '!' flock_decl flock_ex 1 'if flock() does not need to be declared' <<-'EOF'
+ac_test '!' flock_decl flock 1 'if flock() does not need to be declared' <<-'EOF'
 	#define MKSH_INCLUDES_ONLY
 	#include "sh.h"
 	long flock(void);		/* this clashes if defined before */
@@ -1490,10 +1613,11 @@ ac_test '!' revoke_decl revoke 1 'if revoke() does not need to be declared' <<-'
 	long revoke(void);		/* this clashes if defined before */
 	int main(void) { return ((int)revoke()); }
 EOF
-ac_test sys_siglist_decl sys_siglist 1 'if sys_siglist[] does not need to be declared' <<-'EOF'
+ac_test '!' sys_siglist_decl sys_siglist 1 'if sys_siglist[] does not need to be declared' <<-'EOF'
 	#define MKSH_INCLUDES_ONLY
 	#include "sh.h"
-	int main(void) { return (sys_siglist[0][0]); }
+	extern int sys_siglist[5][5][5][5][5];	/* this clashes happily */
+	int main(void) { return (sys_siglist[0][0][0][0][0]); }
 EOF
 CC=$save_CC; LDFLAGS=$save_LDFLAGS; LIBS=$save_LIBS
 
@@ -1501,13 +1625,15 @@ CC=$save_CC; LDFLAGS=$save_LDFLAGS; LIBS=$save_LIBS
 # other checks
 #
 fd='if to use persistent history'
-ac_cache PERSISTENT_HISTORY || test 0 = $HAVE_FLOCK_EX || fv=1
+ac_cache PERSISTENT_HISTORY || case $HAVE_MMAP$HAVE_FLOCK$HAVE_LOCK_FCNTL in
+11*|101) fv=1 ;;
+esac
 test 1 = $fv || check_categories="$check_categories no-histfile"
 ac_testdone
 ac_cppflags
 
 save_CFLAGS=$CFLAGS
-test 1 = $HAVE_CAN_WNOOVERFLOW && CFLAGS="$CFLAGS -Wno-overflow"
+test x1 = x$HAVE_CAN_WNOOVERFLOW && CFLAGS="$CFLAGS -Wno-overflow"
 ac_testn compile_time_asserts_$$ '' 'whether compile-time assertions pass' <<-'EOF'
 	#define MKSH_INCLUDES_ONLY
 	#include "sh.h"
@@ -1555,6 +1681,64 @@ EOF
 CFLAGS=$save_CFLAGS
 eval test 1 = \$HAVE_COMPILE_TIME_ASSERTS_$$ || exit 1
 
+#
+# runtime checks
+# once this is more than one, check if we can do runtime
+# checks (not cross-compiling) first to save on warnings
+#
+$e "${bi}run-time checks follow$ao, please ignore any weird errors"
+
+if ac_testnnd silent_idivwrapv '' '(run-time) whether signed integer division overflows wrap silently' <<-'EOF'
+	#define MKSH_INCLUDES_ONLY
+	#include "sh.h"
+	#ifdef SIGFPE
+	static void fpe_catcher(int) MKSH_A_NORETURN;
+	#endif
+	int main(int ac, char **av) {
+		mksh_ari_t o1, o2, r1, r2;
+
+	#ifdef SIGFPE
+		signal(SIGFPE, fpe_catcher);
+	#endif
+		o1 = ((mksh_ari_t)1 << 31);
+		o2 = -ac;
+		r1 = o1 / o2;
+		r2 = o1 % o2;
+		if (r1 == o1 && r2 == 0) {
+			printf("si");
+			return (0);
+		}
+		printf("no %d %d %d %d %s", (int)o1, (int)o2, (int)r1,
+		    (int)r2, av[0]);
+		return (1);
+	}
+	#ifdef SIGFPE
+	static const char fpe_msg[] = "no, got SIGFPE, what were they smoking?";
+	#define fpe_msglen (sizeof(fpe_msg) - 1)
+	static void fpe_catcher(int sig MKSH_A_UNUSED) {
+		_exit(write(1, fpe_msg, fpe_msglen) == fpe_msglen ? 2 : 3);
+	}
+	#endif
+EOF
+then
+	if test $fv = 0; then
+		echo "| hrm, compiling this failed, but we will just failback"
+	else
+		echo "| running test programme; this will fail if cross-compiling"
+		echo "| in which case we will gracefully degrade to the default"
+		./$tcfn >vv.out 2>&1
+		rv=$?
+		echo "| result: `cat vv.out`"
+		fv=0
+		test $rv = 0 && test x"`cat vv.out`" = x"si" && fv=1
+	fi
+	rmf conftest.c conftest.o ${tcfn}* vv.out
+	ac_testdone
+fi
+ac_cppflags
+
+$e "${bi}end of run-time checks$ao"
+
 #
 # Compiler: Praeprocessor (only if needed)
 #
@@ -1585,7 +1769,8 @@ if test 0 = $HAVE_SYS_SIGNAME; then
 	else
 		$e No list of signal names available via cpp. Falling back...
 	fi
-	sigseen=:
+	sigseenone=:
+	sigseentwo=:
 	echo '#include <signal.h>
 #ifndef NSIG
 #if defined(_NSIG)
@@ -1594,18 +1779,20 @@ if test 0 = $HAVE_SYS_SIGNAME; then
 #define NSIG (SIGMAX+1)
 #endif
 #endif
-mksh_cfg: NSIG' >conftest.c
+int
+mksh_cfg= NSIG
+;' >conftest.c
 	NSIG=`vq "$CPP $CFLAGS $CPPFLAGS $NOWARN conftest.c" | \
-	    grep mksh_cfg: | sed 's/^mksh_cfg:[	 ]*\([0-9x ()+-]*\).*$/\1/'`
+	    sed -n '/^mksh_cfg *=[	 ]*\([0-9x ()+-]*\).*$/s//\1/p'`
 	case $NSIG in
-	*[\ \(\)+-]*) NSIG=`awk "BEGIN { print $NSIG }"` ;;
+	*[\ \(\)+-]*) NSIG=`"$AWK" "BEGIN { print $NSIG }"` ;;
 	esac
 	printf=printf
 	(printf hallo) >/dev/null 2>&1 || printf=echo
 	test $printf = echo || NSIG=`printf %d "$NSIG" 2>/dev/null`
 	$printf "NSIG=$NSIG ... "
-	sigs="ABRT ALRM BUS CHLD CLD CONT DIL EMT FPE HUP ILL INFO INT IO IOT"
-	sigs="$sigs KILL LOST PIPE PROF PWR QUIT RESV SAK SEGV STOP SYS TERM"
+	sigs="INT SEGV ABRT KILL ALRM BUS CHLD CLD CONT DIL EMT FPE HUP ILL"
+	sigs="$sigs INFO IO IOT LOST PIPE PROF PWR QUIT RESV SAK STOP SYS TERM"
 	sigs="$sigs TRAP TSTP TTIN TTOU URG USR1 USR2 VTALRM WINCH XCPU XFSZ"
 	test 1 = $HAVE_CPP_DD && test $NSIG -gt 1 && sigs="$sigs "`vq \
 	    "$CPP $CFLAGS $CPPFLAGS $NOWARN -dD conftest.c" | \
@@ -1613,18 +1800,23 @@ mksh_cfg: NSIG' >conftest.c
 	    sed 's/^\(.*[	 ]SIG\)\([A-Z0-9]*\)\([	 ].*\)$/\2/' | sort`
 	test $NSIG -gt 1 || sigs=
 	for name in $sigs; do
+		case $sigseenone in
+		*:$name:*) continue ;;
+		esac
+		sigseenone=$sigseenone$name:
 		echo '#include <signal.h>' >conftest.c
-		echo mksh_cfg: SIG$name >>conftest.c
+		echo int >>conftest.c
+		echo mksh_cfg= SIG$name >>conftest.c
+		echo ';' >>conftest.c
 		vq "$CPP $CFLAGS $CPPFLAGS $NOWARN conftest.c" | \
-		    grep mksh_cfg: | \
-		    sed 's/^mksh_cfg:[	 ]*\([0-9x]*\).*$/\1:'$name/
-	done | grep -v '^:' | sed 's/:/ /g' | while read nr name; do
+		    sed -n '/^mksh_cfg *=[	 ]*\([0-9x]*\).*$/s//\1:'$name/p
+	done | sed -e '/^:/d' -e 's/:/ /g' | while read nr name; do
 		test $printf = echo || nr=`printf %d "$nr" 2>/dev/null`
 		test $nr -gt 0 && test $nr -le $NSIG || continue
-		case $sigseen in
+		case $sigseentwo in
 		*:$nr:*) ;;
 		*)	echo "		{ \"$name\", $nr },"
-			sigseen=$sigseen$nr:
+			sigseentwo=$sigseentwo$nr:
 			$printf "$name=$nr " >&2
 			;;
 		esac
@@ -1633,25 +1825,32 @@ mksh_cfg: NSIG' >conftest.c
 	$e done.
 fi
 
-addsrcs '!' HAVE_STRLCPY strlcpy.c
+addsrcs -s '!' HAVE_STRLCPY strlcpy.c
 addsrcs USE_PRINTF_BUILTIN printf.c
 test 1 = "$USE_PRINTF_BUILTIN" && add_cppflags -DMKSH_PRINTF_BUILTIN
 test 1 = "$HAVE_CAN_VERB" && CFLAGS="$CFLAGS -verbose"
+test -n "$LDSTATIC" && add_cppflags -DMKSH_OPTSTATIC
+add_cppflags -DMKSH_BUILD_R=409
 
 $e $bi$me: Finished configuration testing, now producing output.$ao
 
 files=
 objs=
 sp=
-case $curdir in
-*\ *)	echo "#!./mksh" >test.sh ;;
-*)	echo "#!$curdir/mksh" >test.sh ;;
+case $tcfn in
+a.exe)	mkshexe=mksh.exe ;;
+*)	mkshexe=mksh ;;
 esac
-cat >>test.sh <<-EOF
+case $curdir in
+*\ *)	mkshshebang="#!./$mkshexe" ;;
+*)	mkshshebang="#!$curdir/$mkshexe" ;;
+esac
+cat >test.sh <<-EOF
+	$mkshshebang
 	LC_ALL=C PATH='$PATH'; export LC_ALL PATH
 	test -n "\$KSH_VERSION" || exit 1
 	set -A check_categories -- $check_categories
-	pflag='$curdir/mksh'
+	pflag='$curdir/$mkshexe'
 	sflag='$srcdir/check.t'
 	usee=0 Pflag=0 uset=0 vflag=0 xflag=0
 	while getopts "C:e:fPp:s:t:v" ch; do case \$ch {
@@ -1722,7 +1921,8 @@ elif test $cm = dragonegg; then
 else
 	emitbc=-c
 fi
-echo set -x >Rebuild.sh
+echo "# work around NeXTstep bug" >Rebuild.sh
+echo set -x >>Rebuild.sh
 for file in $SRCS; do
 	op=`echo x"$file" | sed 's/^x\(.*\)\.c$/\1./'`
 	test -f $file || file=$srcdir/$file
@@ -1747,10 +1947,6 @@ dragonegg|llvm)
 	lobjs=$objs
 	;;
 esac
-case $tcfn in
-a.exe)	mkshexe=mksh.exe ;;
-*)	mkshexe=mksh ;;
-esac
 echo tcfn=$mkshexe >>Rebuild.sh
 echo "$CC $CFLAGS $LDFLAGS -o \$tcfn $lobjs $LIBS $ccpr" >>Rebuild.sh
 echo 'test -f $tcfn || exit 1; size $tcfn' >>Rebuild.sh
@@ -1860,11 +2056,13 @@ exit 0
 === Environment used ===
 
 ==== build environment ====
+AWK				default: awk
 CC				default: cc
 CFLAGS				if empty, defaults to -xO2 or +O2
 				or -O3 -qstrict or -O2, per compiler
 CPPFLAGS			default empty
 LDFLAGS				default empty; added before sources
+LDSTATIC			set this to '-static'; default unset
 LIBS				default empty; added after sources
 				[Interix] default: -lcrypt (XXX still needed?)
 NOWARN				-Wno-error or similar
@@ -1897,8 +2095,11 @@ MKSH_NOPWNAM			skip PAM calls, for -static on eglibc, Solaris
 MKSH_NO_DEPRECATED_WARNING	omit warning when deprecated stuff is run
 MKSH_NO_EXTERNAL_CAT		omit hack to skip cat builtin when flags passed
 MKSH_NO_LIMITS			omit ulimit code
+MKSH_NO_SIGSETJMP		define if sigsetjmp is broken or not available
 MKSH_SMALL			omit some code, optimise hard for size (slower)
-MKSH_S_NOVI			disable Vi editing mode (default if MKSH_SMALL)
+MKSH_S_NOVI=1			disable Vi editing mode (default if MKSH_SMALL)
+MKSH_TYPEDEF_SIG_ATOMIC_T	define to e.g. 'int' if sig_atomic_t is missing
+MKSH_TYPEDEF_SSIZE_T		define to e.g. 'long' if your OS has no ssize_t
 MKSH_UNEMPLOYED			disable job control (but not jobs/co-processes)
 
 === generic installation instructions ===
