@@ -1,7 +1,8 @@
 /*	$OpenBSD: eval.c,v 1.37 2011/10/11 14:32:43 otto Exp $	*/
 
 /*-
- * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+ * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
+ *		 2011, 2012
  *	Thorsten Glaser <tg@mirbsd.org>
  *
  * Provided that these terms and disclaimer and all copyright notices
@@ -22,7 +23,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/eval.c,v 1.110 2011/11/26 00:45:20 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/eval.c,v 1.116 2012/04/06 14:07:52 tg Exp $");
 
 /*
  * string expansion
@@ -375,9 +376,27 @@ expand(const char *cp,	/* input word */
 						sp += slen;
 					switch (stype & 0x17F) {
 					case 0x100 | '#':
+					    {
+						char *beg, *end;
+						mksh_ari_t seed;
+						register uint32_t h;
+
+						beg = wdcopy(sp, ATEMP);
+						end = beg + (wdscan(sp, CSUBST) - sp);
+						end[-2] = EOS;
+						end = wdstrip(beg, 0);
+						afree(beg, ATEMP);
+						evaluate(substitute(end, 0),
+						    &seed, KSH_UNWIND_ERROR, true);
+						/* hash with seed, for now */
+						h = seed;
+						NZATUpdateString(h,
+						    str_val(st->var));
+						NZATFinish(h);
 						x.str = shf_smprintf("%08X",
-						    (unsigned int)hash(str_val(st->var)));
+						    (unsigned int)h);
 						break;
+					}
 					case '0': {
 						char *beg, *mid, *end, *stg;
 						mksh_ari_t from = 0, num = -1, flen, finc = 0;
@@ -863,11 +882,19 @@ expand(const char *cp,	/* input word */
 				fdo = 0;
 				saw_eq = 0;
 				tilde_ok = (f & (DOTILDE|DOASNTILDE)) ? 1 : 0;
-				if (c != 0)
-					Xinit(ds, dp, 128, ATEMP);
-			}
-			if (c == 0)
+				if (c == 0)
+					return;
+				Xinit(ds, dp, 128, ATEMP);
+			} else if (c == 0) {
 				return;
+			} else if (type == XSUB && ctype(c, C_IFS) &&
+			    !ctype(c, C_IFSWS) && Xlength(ds, dp) == 0) {
+				char *p;
+
+				*(p = alloc(1, ATEMP)) = '\0';
+				XPput(*wp, p);
+				type = XSUBMID;
+			}
 			if (word != IFS_NWS)
 				word = ctype(c, C_IFSWS) ? IFS_WS : IFS_NWS;
 		} else {
@@ -888,7 +915,7 @@ expand(const char *cp,	/* input word */
 			if (!quote)
 				switch (c) {
 				case '[':
-				case NOT:
+				case '!':
 				case '-':
 				case ']':
 					/*
@@ -910,10 +937,10 @@ expand(const char *cp,	/* input word */
 						*dp++ = MAGIC;
 					}
 					break;
-				case OBRACE:
+				case '{':
+				case '}':
 				case ',':
-				case CBRACE:
-					if ((f & DOBRACE) && (c == OBRACE ||
+					if ((f & DOBRACE) && (c == '{' /*}*/ ||
 					    (fdo & DOBRACE))) {
 						fdo |= DOBRACE|DOMAGIC;
 						*dp++ = MAGIC;
@@ -1638,7 +1665,7 @@ alt_expand(XPtrV *wp, char *start, char *exp_start, char *end, int fdo)
 	char *p;
 
 	/* search for open brace */
-	for (p = exp_start; (p = strchr(p, MAGIC)) && p[1] != OBRACE; p += 2)
+	for (p = exp_start; (p = strchr(p, MAGIC)) && p[1] != '{' /*}*/; p += 2)
 		;
 	brace_start = p;
 
@@ -1648,9 +1675,9 @@ alt_expand(XPtrV *wp, char *start, char *exp_start, char *end, int fdo)
 		count = 1;
 		for (p += 2; *p && count; p++) {
 			if (ISMAGIC(*p)) {
-				if (*++p == OBRACE)
+				if (*++p == '{' /*}*/)
 					count++;
-				else if (*p == CBRACE)
+				else if (*p == /*{*/ '}')
 					--count;
 				else if (*p == ',' && count == 1)
 					comma = p;
@@ -1681,9 +1708,9 @@ alt_expand(XPtrV *wp, char *start, char *exp_start, char *end, int fdo)
 	count = 1;
 	for (p = brace_start + 2; p != brace_end; p++) {
 		if (ISMAGIC(*p)) {
-			if (*++p == OBRACE)
+			if (*++p == '{' /*}*/)
 				count++;
-			else if ((*p == CBRACE && --count == 0) ||
+			else if ((*p == /*{*/ '}' && --count == 0) ||
 			    (*p == ',' && count == 1)) {
 				char *news;
 				int l1, l2, l3;
