@@ -1,6 +1,6 @@
 /*	$OpenBSD: sh.h,v 1.30 2010/01/04 18:07:11 deraadt Exp $	*/
 /*	$OpenBSD: shf.h,v 1.6 2005/12/11 18:53:51 deraadt Exp $	*/
-/*	$OpenBSD: table.h,v 1.7 2005/12/11 20:31:21 otto Exp $	*/
+/*	$OpenBSD: table.h,v 1.8 2012/02/19 07:52:30 otto Exp $	*/
 /*	$OpenBSD: tree.h,v 1.10 2005/03/28 21:28:22 deraadt Exp $	*/
 /*	$OpenBSD: expand.h,v 1.6 2005/03/30 17:16:37 deraadt Exp $	*/
 /*	$OpenBSD: lex.h,v 1.11 2006/05/29 18:22:24 otto Exp $	*/
@@ -9,7 +9,8 @@
 /*	$OpenBSD: tty.h,v 1.5 2004/12/20 11:34:26 otto Exp $	*/
 
 /*-
- * Copyright © 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+ * Copyright © 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
+ *	       2011, 2012
  *	Thorsten Glaser <tg@mirbsd.org>
  *
  * Provided that these terms and disclaimer and all copyright notices
@@ -151,18 +152,36 @@
 #endif
 
 #ifdef EXTERN
-__RCSID("$MirOS: src/bin/mksh/sh.h,v 1.484.2.11 2011/12/11 18:18:28 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/sh.h,v 1.549 2012/04/14 16:07:48 tg Exp $");
 #endif
-#define MKSH_VERSION "R40 2011/12/11"
+#define MKSH_VERSION "R40 2012/04/14"
 
-/* arithmetics types */
+/* arithmetic types: C implementation */
+#if !HAVE_CAN_INTTYPES
+#if !HAVE_CAN_UCBINTS
+typedef signed int int32_t;
+typedef unsigned int uint32_t;
+#else
+typedef u_int32_t uint32_t;
+#endif
+#endif
+
+/* arithmetic types: shell arithmetics */
+/*
+ * NOTE: these are currently hard-coded to exactly 32 bit, do not change
+ *
+ * TODO: make these configurable, or add 64-bit arithmetic, somehow
+ * (on some operating environments, 64-bit types are needed for some
+ * things, such as the ulimit builtin, to work proper), except mksh
+ * must still be able to run on systems with no native 64-bit integers
+ */
 typedef int32_t mksh_ari_t;
 typedef uint32_t mksh_uari_t;
 
 /* boolean type (no <stdbool.h> deliberately) */
 typedef unsigned char mksh_bool;
 #undef bool
-/* false MUST equal 0 */
+/* false MUST equal the same 0 as written by static storage initialisation */
 #undef false
 #undef true
 /* access macros for boolean type */
@@ -173,6 +192,41 @@ typedef unsigned char mksh_bool;
 /* make any-type into bool or short */
 #define tobool(cond)	((cond) ? true : false)
 
+/* char (octet) type: C implementation */
+#if !HAVE_CAN_INT8TYPE
+#if !HAVE_CAN_UCBINT8
+typedef unsigned char uint8_t;
+#else
+typedef u_int8_t uint8_t;
+#endif
+#endif
+
+/* other standard types */
+
+#if !HAVE_RLIM_T
+typedef unsigned long rlim_t;
+#endif
+
+#if !HAVE_SIG_T
+#undef sig_t
+typedef void (*sig_t)(int);
+#endif
+
+#ifdef MKSH_TYPEDEF_SIG_ATOMIC_T
+typedef MKSH_TYPEDEF_SIG_ATOMIC_T sig_atomic_t;
+#endif
+
+#ifdef MKSH_TYPEDEF_SSIZE_T
+typedef MKSH_TYPEDEF_SSIZE_T ssize_t;
+#endif
+
+/* un-do vendor damage */
+
+#undef BAD		/* AIX defines that somewhere */
+#undef PRINT		/* LynxOS defines that somewhere */
+#undef flock		/* SCO UnixWare defines that to flock64 but ENOENT */
+
+
 #ifndef MKSH_INCLUDES_ONLY
 
 /* extra types */
@@ -191,32 +245,6 @@ struct rusage {
 };
 #endif
 
-#if !HAVE_RLIM_T
-typedef long rlim_t;
-#endif
-
-#if !HAVE_SIG_T
-#undef sig_t
-typedef void (*sig_t)(int);
-#endif
-
-#if !HAVE_CAN_INTTYPES
-#if !HAVE_CAN_UCBINTS
-typedef signed int int32_t;
-typedef unsigned int uint32_t;
-#else
-typedef u_int32_t uint32_t;
-#endif
-#endif
-
-#if !HAVE_CAN_INT8TYPE
-#if !HAVE_CAN_UCBINT8
-typedef unsigned char uint8_t;
-#else
-typedef u_int8_t uint8_t;
-#endif
-#endif
-
 /* extra macros */
 
 #ifndef timerclear
@@ -291,7 +319,6 @@ typedef u_int8_t uint8_t;
 #endif
 #endif
 
-#undef BAD		/* AIX defines that somewhere */
 
 /* OS-dependent additions (functions, variables, by OS) */
 
@@ -307,15 +334,6 @@ extern int getrusage(int, struct rusage *);
 extern int revoke(const char *);
 #endif
 
-#ifdef __ultrix
-/* XXX imake style */
-int strcasecmp(const char *, const char *);
-#endif
-
-#if !HAVE_STRCASESTR
-const char *stristr(const char *, const char *);
-#endif
-
 #if !HAVE_STRLCPY
 size_t strlcpy(char *, const char *, size_t);
 #endif
@@ -331,9 +349,9 @@ extern int __cdecl seteuid(uid_t);
 extern int __cdecl setegid(gid_t);
 #endif
 
-/* remove redundances */
+/* remove redundancies */
 
-#if defined(MirBSD) && (MirBSD >= 0x08A8)
+#if defined(MirBSD) && (MirBSD >= 0x08A8) && !defined(MKSH_OPTSTATIC)
 #define MKSH_mirbsd_wcwidth
 #define utf_wcwidth(i) wcwidth((__WCHAR_TYPE__)i)
 extern int wcwidth(__WCHAR_TYPE__);
@@ -365,11 +383,10 @@ extern int wcwidth(__WCHAR_TYPE__);
 /*
  * Make MAGIC a char that might be printed to make bugs more obvious, but
  * not a char that is used often. Also, can't use the high bit as it causes
- * portability problems (calling strchr(x, 0x80|'x') is error prone).
+ * portability problems (calling strchr(x, 0x80 | 'x') is error prone).
  */
 #define MAGIC		(7)	/* prefix for *?[!{,} during expand */
 #define ISMAGIC(c)	((unsigned char)(c) == MAGIC)
-#define NOT		'!'	/* might use ^ (ie, [!...] vs [^..]) */
 
 #define LINE		4096	/* input line size */
 
@@ -425,6 +442,16 @@ char *ucstrstr(char *, const char *);
 #define mkssert(e)	((void)0)
 #endif
 
+#if (!defined(MKSH_BUILDMAKEFILE4BSD) && !defined(MKSH_BUILDSH)) || (MKSH_BUILD_R != 409)
+#error Must run Build.sh to compile this.
+int
+im_sorry_dave(void)
+{
+	/* I’m sorry, Dave. I’m afraid I can’t do that. */
+	return (thiswillneverbedefinedIhope());
+}
+#endif
+
 /* use this ipv strchr(s, 0) but no side effects in s! */
 #define strnul(s)	((s) + strlen(s))
 
@@ -466,10 +493,6 @@ char *ucstrstr(char *, const char *);
 } while (/* CONSTCOND */ 0)
 #endif
 
-#if HAVE_STRCASESTR
-#define stristr(b,l)		((const char *)strcasestr((b), (l)))
-#endif
-
 #ifdef MKSH_SMALL
 #ifndef MKSH_CONSERVATIVE_FDS
 #define MKSH_CONSERVATIVE_FDS	/* defined */
@@ -512,11 +535,6 @@ char *ucstrstr(char *, const char *);
 #define free_ossetmode(p)	free(p)
 #endif
 
-#if !HAVE_MKSTEMP
-/* tempnam(3) -> free(3) */
-#define free_ostempnam(p)	free(p)
-#endif
-
 #ifdef NO_PATH_MAX
 /* GNU libc: get_current_dir_name(3) -> free(3) */
 #define free_gnu_gcdn(p)	free(p)
@@ -555,6 +573,16 @@ enum sh_flag {
 /*
  * parsing & execution environment
  */
+#ifdef MKSH_NO_SIGSETJMP
+#define kshjmp_buf	jmp_buf
+#define kshsetjmp(jbuf)	_setjmp(jbuf)
+#define kshlongjmp	_longjmp
+#else
+#define kshjmp_buf	sigjmp_buf
+#define kshsetjmp(jbuf)	sigsetjmp((jbuf), 0)
+#define kshlongjmp	siglongjmp
+#endif
+
 extern struct env {
 	ALLOC_ITEM alloc_INT;	/* internal, do not touch */
 	Area area;		/* temporary allocation area */
@@ -562,9 +590,9 @@ extern struct env {
 	struct block *loc;	/* local variables and functions */
 	short *savefd;		/* original redirected fds */
 	struct temp *temps;	/* temp files */
-	sigjmp_buf jbuf;	/* long jump back to env creator */
-	short type;		/* environment type - see below */
-	short flags;		/* EF_* */
+	kshjmp_buf jbuf;	/* long jump back to env creator */
+	uint8_t type;		/* environment type - see below */
+	uint8_t flags;		/* EF_* */
 } *e;
 
 /* struct env.type values */
@@ -583,12 +611,12 @@ extern struct env {
 #define EF_FAKE_SIGDIE	BIT(2)	/* hack to get info from unwind to quitenv */
 
 /* Do breaks/continues stop at env type e? */
-#define STOP_BRKCONT(t)	((t) == E_NONE || (t) == E_PARSE \
-			 || (t) == E_FUNC || (t) == E_INCL)
+#define STOP_BRKCONT(t)	((t) == E_NONE || (t) == E_PARSE || \
+			    (t) == E_FUNC || (t) == E_INCL)
 /* Do returns stop at env type e? */
 #define STOP_RETURN(t)	((t) == E_FUNC || (t) == E_INCL)
 
-/* values for siglongjmp(e->jbuf, 0) */
+/* values for kshlongjmp(e->jbuf, i) */
 #define LRETURN	1	/* return statement */
 #define LEXIT	2	/* exit statement */
 #define LERROR	3	/* errorf() called */
@@ -671,18 +699,21 @@ EXTERN const char Tgbuiltin[] E_INIT("=builtin");
 EXTERN const char T_function[] E_INIT(" function");
 #define Tfunction	(T_function + 1)	/* "function" */
 
-enum temp_type {
-	TT_HEREDOC_EXP,	/* expanded heredoc */
-	TT_HIST_EDIT	/* temp file used for history editing (fc -e) */
-};
-typedef enum temp_type Temp_type;
+typedef uint8_t Temp_type;
+/* expanded heredoc */
+#define TT_HEREDOC_EXP	0
+/* temp file used for history editing (fc -e) */
+#define TT_HIST_EDIT	1
+
 /* temp/heredoc files. The file is removed when the struct is freed. */
 struct temp {
 	struct temp *next;
 	struct shf *shf;
-	char *name;
-	int pid;	/* pid of process parsed here-doc */
+	/* pid of process parsed here-doc */
+	pid_t pid;
 	Temp_type type;
+	/* actually longer: name (variable length) */
+	char tffn[3];
 };
 
 /*
@@ -692,6 +723,9 @@ struct temp {
 #define shl_spare	(&shf_iob[0])	/* for c_read()/c_print() */
 #define shl_stdout	(&shf_iob[1])
 #define shl_out		(&shf_iob[2])
+#ifdef DF
+#define shl_dbg		(&shf_iob[3])	/* for DF() */
+#endif
 EXTERN bool shl_stdout_ok;
 
 /*
@@ -851,15 +885,7 @@ EXTERN char	*current_wd;
 #define MIN_COLS	(2 + MIN_EDIT_SPACE + 3)
 #define MIN_LINS	3
 EXTERN mksh_ari_t x_cols E_INIT(80);	/* tty columns */
-EXTERN mksh_ari_t x_lins E_INIT(-1);	/* tty lines */
-
-/* These to avoid bracket matching problems */
-#define OPAREN	'('
-#define CPAREN	')'
-#define OBRACK	'['
-#define CBRACK	']'
-#define OBRACE	'{'
-#define CBRACE	'}'
+EXTERN mksh_ari_t x_lins E_INIT(24);	/* tty lines */
 
 
 /* Determine the location of the system (common) profile */
@@ -876,7 +902,7 @@ EXTERN mksh_ari_t x_lins E_INIT(-1);	/* tty lines */
 
 
 /* Used by v_evaluate() and setstr() to control action when error occurs */
-#define KSH_UNWIND_ERROR	0	/* unwind the stack (longjmp) */
+#define KSH_UNWIND_ERROR	0	/* unwind the stack (kshlongjmp) */
 #define KSH_RETURN_ERROR	1	/* return 1/0 for success/failure */
 
 /*
@@ -1448,11 +1474,9 @@ EXTERN YYSTYPE yylval;		/* result from yylex */
 EXTERN struct ioword *heres[HERES], **herep;
 EXTERN char ident[IDENT+1];
 
-#define HISTORYSIZE	500	/* size of saved history */
-
-EXTERN char **history;	/* saved commands */
-EXTERN char **histptr;	/* last history item */
-EXTERN int histsize;	/* history size */
+EXTERN char **history;		/* saved commands */
+EXTERN char **histptr;		/* last history item */
+EXTERN mksh_ari_t histsize;	/* history size */
 
 /* user and system time of last j_waitjed job */
 EXTERN struct timeval j_usrtime, j_systime;
@@ -1467,34 +1491,54 @@ EXTERN struct timeval j_usrtime, j_systime;
 } while (/* CONSTCOND */ 0)
 
 
-/* Bob Jenkins' one-at-a-time hash, with better start value */
-#define oaat1_init_impl(h) do {						\
-	(h) = 0x100;							\
+/* NZAT/NZAAT hashes based on Bob Jenkins' one-at-a-time hash */
+
+/* From: src/kern/include/nzat.h,v 1.2 2011/07/18 00:35:40 tg Exp $ */
+
+#define NZATInit(h) do {					\
+	(h) = 0;						\
 } while (/* CONSTCOND */ 0)
-#define oaat1_addmem_impl(h, buf, len) do {				\
-	register const uint8_t *oaat1_addmem_p = (const void *)(buf);	\
-	register size_t oaat1_addmem_n = (len);				\
-									\
-	while (oaat1_addmem_n--) {					\
-		(h) += *oaat1_addmem_p++;				\
-		(h) += (h) << 10;					\
-		(h) ^= (h) >> 6;					\
-	}								\
+
+#define NZATUpdateByte(h,b) do {				\
+	(h) += (uint8_t)(b);					\
+	++(h);							\
+	(h) += (h) << 10;					\
+	(h) ^= (h) >> 6;					\
 } while (/* CONSTCOND */ 0)
-#define oaat1_addstr_impl(h, s) do {					\
-	register const uint8_t *oaat1_addstr_p = (const void *)(s);	\
-	register uint8_t oaat1_addstr_c;				\
-									\
-	while ((oaat1_addstr_c = *oaat1_addstr_p++)) {			\
-		h += oaat1_addstr_c;					\
-		(h) += (h) << 10;					\
-		(h) ^= (h) >> 6;					\
-	}								\
+
+#define NZATUpdateMem(h,p,z) do {				\
+	register const uint8_t *NZATUpdateMem_p;		\
+	register size_t NZATUpdateMem_z = (z);			\
+								\
+	NZATUpdateMem_p = (const void *)(p);			\
+	while (NZATUpdateMem_z--)				\
+		NZATUpdateByte((h), *NZATUpdateMem_p++);	\
 } while (/* CONSTCOND */ 0)
-#define oaat1_fini_impl(h) do {						\
-	(h) += (h) << 3;						\
-	(h) ^= (h) >> 11;						\
-	(h) += (h) << 15;						\
+
+#define NZATUpdateString(h,s) do {				\
+	register const char *NZATUpdateString_s;		\
+	register uint8_t NZATUpdateString_c;			\
+								\
+	NZATUpdateString_s = (const void *)(s);			\
+	while ((NZATUpdateString_c = *NZATUpdateString_s++))	\
+		NZATUpdateByte((h), NZATUpdateString_c);	\
+} while (/* CONSTCOND */ 0)
+
+/* not zero after termination */
+#define NZATFinish(h) do {					\
+	if ((h) == 0)						\
+		++(h);						\
+	else							\
+		NZAATFinish(h);					\
+} while (/* CONSTCOND */ 0)
+
+/* NULs zählen an allen Teilen */
+#define NZAATFinish(h) do {					\
+	(h) += (h) << 10;					\
+	(h) ^= (h) >> 6;					\
+	(h) += (h) << 3;					\
+	(h) ^= (h) >> 11;					\
+	(h) += (h) << 15;					\
 } while (/* CONSTCOND */ 0)
 
 
@@ -1607,17 +1651,19 @@ void histsave(int *, const char *, bool, bool);
 bool histsync(void);
 #endif
 int c_fc(const char **);
-void sethistsize(int);
+void sethistsize(mksh_ari_t);
 #if HAVE_PERSISTENT_HISTORY
 void sethistfile(const char *);
 #endif
+#if !MKSH_S_NOVI
 char **histpos(void);
 int histnum(int);
+#endif
 int findhist(int, int, const char *, int);
 char **hist_get_newest(bool);
 void inittraps(void);
 void alarm_init(void);
-Trap *gettrap(const char *, int);
+Trap *gettrap(const char *, bool);
 void trapsig(int);
 void intrcheck(void);
 int fatal_trap_check(void);
@@ -1631,6 +1677,10 @@ int block_pipe(void);
 void restore_pipe(int);
 int setsig(Trap *, sig_t, int);
 void setexecsig(Trap *, int);
+#if HAVE_FLOCK || HAVE_LOCK_FCNTL
+void mksh_lockfd(int);
+void mksh_unlkfd(int);
+#endif
 /* jobs.c */
 void j_init(void);
 void j_exit(void);
@@ -1705,7 +1755,12 @@ void coproc_write_close(int);
 int coproc_getfd(int, const char **);
 void coproc_cleanup(int);
 struct temp *maketemp(Area *, Temp_type, struct temp **);
-void ktinit(Area *, struct table *, uint8_t);
+void ktinit_real(Area *, struct table *, uint8_t);
+#ifdef MKSH_SMALL
+#define ktinit(ap, tp, s80, s66) ktinit_real((ap), (tp), (s80))
+#else
+#define ktinit(ap, tp, s80, s66) ktinit_real((ap), (tp), (s66))
+#endif
 struct tbl *ktscan(struct table *, const char *, uint32_t, struct tbl ***);
 /* table, name (key) to search for, hash(n) */
 #define ktsearch(tp, s, h) ktscan((tp), (s), (h), NULL)
@@ -1714,6 +1769,10 @@ struct tbl *ktenter(struct table *, const char *, uint32_t);
 void ktwalk(struct tstate *, struct table *);
 struct tbl *ktnext(struct tstate *);
 struct tbl **ktsort(struct table *);
+#ifdef DF
+void DF(const char *, ...)
+    MKSH_A_FORMAT(__printf__, 1, 2);
+#endif
 /* misc.c */
 void setctypes(const char *, int);
 void initctypes(void);
@@ -1728,7 +1787,7 @@ int has_globbing(const char *, const char *);
 int xstrcmp(const void *, const void *);
 void ksh_getopt_reset(Getopt *, int);
 int ksh_getopt(const char **, Getopt *, const char *);
-void print_value_quoted(const char *);
+void print_value_quoted(struct shf *, const char *);
 char *quote_value(const char *);
 void print_columns(struct shf *, int,
     char *(*)(char *, size_t, int, const void *),
@@ -1798,6 +1857,7 @@ void tfree(struct op *, Area *);
 void dumpchar(struct shf *, int);
 void dumptree(struct shf *, struct op *);
 void dumpwdvar(struct shf *, const char *);
+void dumpioact(struct shf *shf, struct op *t);
 void vistree(char *, size_t, struct op *)
     MKSH_A_BOUNDED(__string__, 1, 2);
 void fpFUNCTf(struct shf *, int, bool, const char *, struct op *);
@@ -1826,6 +1886,7 @@ size_t array_ref_len(const char *);
 char *arrayname(const char *);
 mksh_uari_t set_array(const char *, bool, const char **);
 uint32_t hash(const void *);
+mksh_ari_t rndget(void);
 void rndset(long);
 
 enum Test_op {
