@@ -1,7 +1,8 @@
 /*	$OpenBSD: var.c,v 1.34 2007/10/15 02:16:35 deraadt Exp $	*/
 
 /*-
- * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+ * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
+ *		 2011, 2012
  *	Thorsten Glaser <tg@mirbsd.org>
  *
  * Provided that these terms and disclaimer and all copyright notices
@@ -26,7 +27,7 @@
 #include <sys/sysctl.h>
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/var.c,v 1.130.2.6 2011/12/11 18:18:30 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/var.c,v 1.146 2012/04/14 14:35:13 tg Exp $");
 
 /*-
  * Variables
@@ -74,8 +75,8 @@ newblock(void)
 		l->argv = e->loc->argv;
 	}
 	l->exit = l->error = NULL;
-	ktinit(&l->area, &l->vars, 0);
-	ktinit(&l->area, &l->funs, 0);
+	ktinit(&l->area, &l->vars, 0, 0);
+	ktinit(&l->area, &l->funs, 0, 0);
 	l->next = e->loc;
 	e->loc = l;
 }
@@ -130,8 +131,8 @@ initvar(void)
 	struct tbl *tp;
 
 	ktinit(APERM, &specials,
-	    /* currently 12 specials -> 80% of 16 (2^4) */
-	    4);
+	    /* currently 12 specials */
+	    /* 80% of 16 = 2^4 */ 4, /* 66% of 32 = 2^5 */ 5);
 	while (i < V_MAX - 1) {
 		tp = ktenter(&specials, initvar_names[i],
 		    hash(initvar_names[i]));
@@ -409,7 +410,7 @@ setstr(struct tbl *vq, const char *s, int error_ok)
 
 	error_ok &= ~0x4;
 	if ((vq->flag & RDONLY) && !no_ro_check) {
-		warningf(true, "%s: %s", vq->name, "is read only");
+		warningf(true, "read-only: %s", vq->name);
 		if (!error_ok)
 			errorfxz(2);
 		return (0);
@@ -487,26 +488,11 @@ getint(struct tbl *vp, mksh_ari_t *nump, bool arith)
 	base = 10;
 	num = 0;
 	neg = 0;
-#ifdef MKSH_DISABLE_DEPRECATED
 	if (arith && s[0] == '0' && (s[1] | 0x20) == 'x') {
 		s += 2;
 		base = 16;
 		have_base = true;
 	}
-#else
-	if (arith && *s == '0' && *(s+1)) {
-		s++;
-		if (*s == 'x' || *s == 'X') {
-			s++;
-			base = 16;
-		} else if (vp->flag & ZEROFIL) {
-			while (*s == '0')
-				s++;
-		} else
-			base = 8;
-		have_base = true;
-	}
-#endif
 	while ((c = *s++)) {
 		if (c == '-') {
 			neg++;
@@ -709,7 +695,6 @@ typeset(const char *var, uint32_t set, uint32_t clr, int field, int base)
 	if (val == var)
 		return (NULL);
 	mkssert(var != NULL);
-	mkssert(*var != 0);
 	if (*val == '[') {
 		if (set_refflag != SRF_NOP)
 			errorf("%s: %s", var,
@@ -810,7 +795,7 @@ typeset(const char *var, uint32_t set, uint32_t clr, int field, int base)
 	if ((vpbase->flag&RDONLY) &&
 	    (val || clr || (set & ~EXPORT)))
 		/* XXX check calls - is error here ok by POSIX? */
-		errorfx(2, "%s: %s", tvar, "is read only");
+		errorfx(2, "read-only: %s", tvar);
 	afree(tvar, ATEMP);
 
 	/* most calls are with set/clr == 0 */
@@ -1109,11 +1094,7 @@ getspec(struct tbl *vp)
 			return;
 		break;
 	case V_RANDOM:
-		/*
-		 * this is the same Linear Congruential PRNG as Borland
-		 * C/C++ allegedly uses in its built-in rand() function
-		 */
-		i = ((lcg_state = 22695477 * lcg_state + 1) >> 16) & 0x7FFF;
+		i = rndget();
 		break;
 	case V_HISTSIZE:
 		i = histsize;
@@ -1133,7 +1114,6 @@ getspec(struct tbl *vp)
 		 * and the window is then resized, the app won't
 		 * see the change cause the environ doesn't change.
 		 */
-		change_winsz();
 		i = st == V_COLUMNS ? x_cols : x_lins;
 		break;
 	default:
@@ -1220,7 +1200,7 @@ setspec(struct tbl *vp)
 		getopts_reset((int)i);
 		break;
 	case V_HISTSIZE:
-		sethistsize((int)i);
+		sethistsize(i);
 		break;
 	case V_COLUMNS:
 		if (i >= MIN_COLS)
@@ -1401,7 +1381,7 @@ set_array(const char *var, bool reset, const char **vals)
 
 	/* Note: AT&T ksh allows set -A but not set +A of a read-only var */
 	if ((vp->flag&RDONLY))
-		errorfx(2, "%s: %s", ccp, "is read only");
+		errorfx(2, "read-only: %s", ccp);
 	/* This code is quite non-optimal */
 	if (reset) {
 		/* trash existing values and attributes */
@@ -1459,16 +1439,6 @@ set_array(const char *var, bool reset, const char **vals)
 void
 change_winsz(void)
 {
-	if (x_lins < 0) {
-		/* first time initialisation */
-#ifdef TIOCGWINSZ
-		if (tty_fd < 0)
-			/* non-FTALKING, try to get an fd anyway */
-			tty_init(true, false);
-#endif
-		x_cols = -1;
-	}
-
 #ifdef TIOCGWINSZ
 	/* check if window size has changed */
 	if (tty_fd >= 0) {
@@ -1499,20 +1469,30 @@ hash(const void *s)
 {
 	register uint32_t h;
 
-	oaat1_init_impl(h);
-	oaat1_addstr_impl(h, s);
-	oaat1_fini_impl(h);
+	NZATInit(h);
+	NZATUpdateString(h, s);
+	NZATFinish(h);
 	return (h);
 }
 
+mksh_ari_t
+rndget(void)
+{
+	/*
+	 * this is the same Linear Congruential PRNG as Borland
+	 * C/C++ allegedly uses in its built-in rand() function
+	 */
+	return (((lcg_state = 22695477 * lcg_state + 1) >> 16) & 0x7FFF);
+}
+
 void
 rndset(long v)
 {
 	register uint32_t h;
 
-	oaat1_init_impl(h);
-	oaat1_addmem_impl(h, &lcg_state, sizeof(lcg_state));
-	oaat1_addmem_impl(h, &v, sizeof(v));
+	NZATInit(h);
+	NZATUpdateMem(h, &lcg_state, sizeof(lcg_state));
+	NZATUpdateMem(h, &v, sizeof(v));
 
 #if defined(arc4random_pushb_fast) || defined(MKSH_A4PB)
 	/*
@@ -1521,16 +1501,16 @@ rndset(long v)
 	 * user requested us to use the old functions
 	 */
 	lcg_state = h;
-	oaat1_fini_impl(lcg_state);
+	NZAATFinish(lcg_state);
 #if defined(arc4random_pushb_fast)
 	arc4random_pushb_fast(&lcg_state, sizeof(lcg_state));
 	lcg_state = arc4random();
 #else
 	lcg_state = arc4random_pushb(&lcg_state, sizeof(lcg_state));
 #endif
-	oaat1_addmem_impl(h, &lcg_state, sizeof(lcg_state));
+	NZATUpdateMem(h, &lcg_state, sizeof(lcg_state));
 #endif
 
-	oaat1_fini_impl(h);
+	NZAATFinish(h);
 	lcg_state = h;
 }
